<?php

namespace Drupal\lingotek_overrides\Remote;

use Drupal\lingotek\Remote\LingotekHttp as BaseLingotekHttp;
use GuzzleHttp\RequestOptions;

/**
 * Decorates the lingotek.http_client service.
 *
 * Overrides the ::patch() method to allow multipart content.
 *
 * @package Drupal\lingotek_overrides\Remote
 */
class LingotekHttp extends BaseLingotekHttp {

  /**
   * {@inheritdoc}
   */
  public function patch($path, $args = [], $use_multipart = FALSE) {
    $options = [];

    if (count($args) && $use_multipart) {
      $multipart = [];
      foreach ($args as $name => $contents) {
        $multipart[] = ['name' => $name, 'contents' => $contents];
      }
      $options[RequestOptions::MULTIPART] = $multipart;
    }
    elseif (count($args) && !$use_multipart) {
      $options[RequestOptions::FORM_PARAMS] = $args;
    }

    return $this->httpClient->patch($this->getBaseUrl() . $path,
      [
        RequestOptions::HEADERS => $this->getDefaultHeaders() +
          // Let the post method masquerade as a PATCH.
          ['X-HTTP-Method-Override' => 'PATCH'],
      ] + $options
    );
  }

}
